#!/bin/bash

### 變數區 ###
input_account=""
input_date=""
jet=""
#--------------------------------

check_cpuser() {
# 執行 WHM API 命令，將結果存儲在變數中
account_result=$(whmapi1 --output=jsonpretty listaccts search=${1} searchtype=user)


# 檢查結果中是否包含 "acct" 鍵，表示有帳號信息
if grep -q '"acct"' <<< "$account_result"; then
    input_account=$1
else
    echo "此主機內沒有此cPanel 帳號!"
    exit
fi
}

# 定義日期檢查函數
check_date_format() {
    local date="$1"
    local regex="^(19|20)[0-9]{2}-(0[1-9]|1[0-2])-(0[1-9]|[12][0-9]|3[01])$"

    if [[ $date =~ $regex ]]; then
 	input_date=$date
    else
        echo "日期格式錯誤"
	exit
    fi
}


# 使用 whmapi1 列出所有經銷帳號
response=$(whmapi1 --output=jsonpretty listresellers)

# 使用 jq 解析 JSON 回應並檢查 data.reseller 是否為空
is_empty=$(echo "$response" | jq -r '.data.reseller | if length == 0 then "true" else "false" end')

# 判斷 data.reseller 是否為空並輸出結果
if [ "$is_empty" == "true" ]; then
    echo "此主機非經銷主機，不執行此程式!"
    exit
else
    :
fi



if [ -z "$1" ]; then
     echo "請於指令後面加上經銷主帳號名稱"
     echo "例如: /opt/cpinst/steven_script/reseller_restore_resource_backup_config.sh cityweb"
     exit
 else
     check_cpuser $1
 fi


if [ -z "$2" ]; then
     echo "請於指令後面加上經銷主帳號名稱以及還原備份檔日期"	
     echo "例如: /opt/cpinst/steven_script/reseller_restore_resource_backup_config.sh cityweb 2024-05-27"
     exit
 else
     check_date_format $2
 fi


# 當輸入變數皆符合條件時
if [ -n "$input_account" ] && [ -n "$input_date" ]; then

     if [ -e /usr/bin/jetbackup ]; then
	 jet="4"
     fi
     if [ -e /usr/bin/jetbackup5 ]; then
         jet="5"
     fi

     # 列出所有 owner 是輸入值的清單
     reseller_all_accounts=$(whmapi1 --output=jsonpretty listaccts | jq --arg account $input_account '.data.acct[] | select(.owner == $account) | .user' | sed 's/\"//g')
     
     if [ -z "$reseller_all_accounts" ]; then
         echo "請輸入經銷主帳號!"
	 exit
     fi


     for reseller_account in $reseller_all_accounts; do
##================================================================================================	   
         if [ "$jet" == "4" ]; then
		 # 找出指定日期的snap 編號 (每個帳號備份編號都不太一樣)
             snap_name=$(ls -lt --time-style=long-iso /backup/jetbackup/jetbackup_1_1_*/$reseller_account/snap.*/jetbackup.index | grep $input_date | awk -F'/' '{for (i=1; i<=NF; i++) if ($i ~ /^snap\./) print $i}')
             if [ -z "$snap_name" ]; then
                 echo "輸入日期不存在備份檔"
                 exit
             else
                 :
             fi

             # 磁碟空間值 (已轉換 MB -> Bytes)
             diskquota_value=$(expr $(cat /backup/jetbackup/jetbackup_1_1_*/${reseller_account}/${snap_name}/quota) \* 1024 \* 1024)
             # 流量值
             bandwidth_value=$(cat /backup/jetbackup/jetbackup_1_1_*/${reseller_account}/${snap_name}/cp/${reseller_account} | grep BWLIMIT |awk -F 'BWLIMIT=' '{print $2}')

	     if [ "$diskquota_value" == "0" ] || [ "$bandwidth_value" == "0" ]; then
		  continue
	     fi

	     if [ -n "$diskquota_value" ] && [ -n "$bandwidth_value" ]; then
                 whmapi1 --output=jsonpretty modifyacct user=${reseller_account} QUOTA=$diskquota_value   BWLIMIT=$bandwidth_value
	     fi	 
         fi

##================================================================================================

         if [ "$jet" == "5" ]; then
         # 找出指定日期的snap 編號
         snap_name=$(ls -lt --time-style=long-iso /backup/jetbackup/jetbackup_1_1_*/${reseller_account}_*/snap.*/jetbackup.index | grep $input_date | awk -F'/' '{for (i=1; i<=NF; i++) if ($i ~ /^snap\./) print $i}')
         if [ -z "$snap_name" ]; then
             echo "輸入日期不存在備份檔"
             exit
         else
             :
         fi

         # 磁碟空間值 (已轉換 MB -> Bytes)
         diskquota_value=$(expr $(cat /backup/jetbackup/jetbackup_1_1_*/${reseller_account}_*/${snap_name}/config/quota) \* 1024 \* 1024)
         # 流量值
         bandwidth_value=$(cat /backup/jetbackup/jetbackup_1_1_*/${reseller_account}_*/${snap_name}/config/cp/${reseller_account} | grep BWLIMIT |awk -F 'BWLIMIT=' '{print $2}')
	 
	 if [ "$diskquota_value" == "0" ] || [ "$bandwidth_value" == "0" ]; then
             continue
         fi

	 if [ -n "$diskquota_value" ] && [ -n "$bandwidth_value" ]; then
             whmapi1 --output=jsonpretty modifyacct user=${reseller_account} QUOTA=$diskquota_value   BWLIMIT=$bandwidth_value
         fi

         fi
     done        
else
    echo "未輸入 cPanel帳號 或是 時間日期!"
    exit
fi
